library(shiny)
library(R.matlab)
library(igraph)

env <- globalenv()

Visualize_Clust <- function(Gene_Names, GRNPath, TFPath, interact, clustering){  
  
  TF <- unlist(read.table(TFPath))
  
  Ainferred <- readMat(GRNPath)
  Ainferred <- Ainferred$Network
  
  nb_col <- length(unique(clustering))
  C <- unique(clustering)
  Nclust <- clustering
  
  for (i in 1:nb_col){
    Nclust[which(clustering==C[i])] = i
  }  
  clustering <- Nclust
  
  Colors <- rainbow(nb_col)
  Colors <- c('red', 'blue', 'forestgreen', 'gold', 'plum2', 'turquoise1' , 'black')
  Col_Lab <- Colors[clustering]   
  
  
  if (length(which(Ainferred!=0))!=0){
    
    rem_row <- which(rowSums(Ainferred)==0)
    rem_col <- which(colSums(Ainferred)==0)
    rem <- intersect(rem_row, rem_col)
    sel <- setdiff(1:length(Gene_Names), rem)
    
    Aclean <- Ainferred[sel,sel]
    clust <- Col_Lab[sel]
    
    
    Clean_names <- Gene_Names[sel]
    
    write.table(Clean_names, 'results/Genes_GRN.txt', row.names=F, col.names=F)
    Clean_TF <- which(sel%in%TF)
    cat('This graph contains', length(Clean_TF), 'TFs.\n')
    
    
    ResPerTF <- matrix(ncol = length(Clean_TF)+1, nrow = max(colSums(Aclean))+1)
    ResPerTF[,1] <- c('TF', paste(rep('Target_', max(colSums(Aclean))), 1:max(colSums(Aclean)), sep=''))
    ResPerTF[1,2:(length(Clean_TF)+1)] <- unlist(Clean_names[Clean_TF])
    
    for (k in 1:length(Clean_TF)) {
      Connect <- which(Aclean[,Clean_TF[k]] != 0)
      ResPerTF[2:(length(Connect)+1),k+1] <- unlist(Clean_names[Connect])
    }
    write.table(ResPerTF,'results/ListInteractionsPerTF.txt', quote=F, col.names=F, row.names = F, sep='\t', na = "")
    Ginferred <- graph.adjacency(Aclean, weighted=T)

  } else {
    return(NULL)
    
  }
  
  V(Ginferred)$size <- 5
  V(Ginferred)[Clean_TF]$size <- 8
  V(Ginferred)$color <- clust
  
  # par(bg = '#fcf7f7')
  if (interact==TRUE) {tkplot(Ginferred, edge.arrow.size=0, edge.width = 1, vertex.label = Clean_names)} 
  plot(Ginferred, edge.arrow.size=0, edge.width = 1, vertex.label = Clean_names, vertex.label.cex=1.5, vertex.label.dist=0.5)
  col = Colors
  legende <- paste('Cluster', 1:nb_col, sep=' ')
  pch = rep(20,nb_col)
  pt = rep(2, nb_col)
 # legend(1, 1.3, legende, col=col, pch = pch, pt.cex=pt)
  return(Ginferred)
}
Visualize <- function(Gene_Names, GRNPath, TFPath, interact, clustering){  
  
  TF <- unlist(read.table(TFPath))
  
  Ainferred <- readMat(GRNPath)
  Ainferred <- Ainferred$Network
  

  if (length(which(Ainferred!=0))!=0){
    
    rem_row <- which(rowSums(Ainferred)==0)
    rem_col <- which(colSums(Ainferred)==0)
    rem <- intersect(rem_row, rem_col)
    sel <- setdiff(1:length(Gene_Names), rem)
    
    Aclean <- Ainferred[sel,sel]
    
    Clean_names <- Gene_Names[sel]
    
    write.table(Clean_names, 'results/Genes_GRN.txt', row.names=F, col.names=F)
    Clean_TF <- which(sel%in%TF)
    cat('This graph contains', length(Clean_TF), 'TFs.\n')
    
    
    ResPerTF <- matrix(ncol = length(Clean_TF)+1, nrow = max(colSums(Aclean))+1)
    ResPerTF[,1] <- c('TF', paste(rep('Target_', max(colSums(Aclean))), 1:max(colSums(Aclean)), sep=''))
    ResPerTF[1,2:(length(Clean_TF)+1)] <- unlist(Clean_names[Clean_TF])
    
    for (k in 1:length(Clean_TF)) {
      Connect <- which(Aclean[,Clean_TF[k]] != 0)
      ResPerTF[2:(length(Connect)+1),k+1] <- unlist(Clean_names[Connect])
    }
    write.table(ResPerTF,'results/ListInteractionsPerTF.txt', quote=F, col.names=F, row.names = F, sep='\t', na = "")
    Ginferred <- graph.adjacency(Aclean, weighted=T)
    
  } else {
    return(NULL)
    
  }
  
  V(Ginferred)$size <- 5
  V(Ginferred)[Clean_TF]$size <- 8
  
  # par(bg = '#fcf7f7')
  if (interact==TRUE) {tkplot(Ginferred, edge.arrow.size=0, edge.width = 1, vertex.label = Clean_names)} 
  plot(Ginferred, edge.arrow.size=0, edge.width = 1, vertex.label = Clean_names, vertex.label.cex=1.5, vertex.label.dist=0.5)
  return(Ginferred)
}

# Define server logic required to draw a histogram
shinyServer(function(input, output, session) {
  
  getData <- reactive({
    input$data_input
    inFile <- input$data_input
    if(is.null(inFile)) return(NULL)
    else {
      GEMpath <- paste('../data/', inFile$name, sep='')
      GEM <- read.table( GEMpath, header = F)
      nb_nodes <- dim(GEM)[1]
      assign('nb_nodes', nb_nodes, env)
      assign('GEMpath', GEMpath, env)
      assign('FileName', inFile$name, env)
    }
  })
  
  output$fileUploaded <- reactive({
    return(!is.null(getData()))
  })
  outputOptions(output, 'fileUploaded', suspendWhenHidden=FALSE)
  
  output$Download <- renderPrint({
    if (!is.null(getData()))
      cat(paste('File', FileName, 'successfully downloaded!\n\nClick the Weight Computation button to get the adjacency matrix, this may take a while! '))
  })
  
  compWeights <- reactive({
    if(input$weight_comp==0) return(NULL)
    else{
      system(paste('../distrib/Generate_Weights.exe', GEMpath)) 
      return(1)
    }
  })
  output$WeightsComputed <- reactive({
    return(!is.null(compWeights()))
  })
  outputOptions(output, 'WeightsComputed', suspendWhenHidden=FALSE)
  
  
  output$Weights <- renderPrint({
    if (!is.null(compWeights()))
      cat('Adjacency matrix created!\n\nClick the TF List button to load the list of transcription factors.')
  })
  
  TFloading <- reactive({
    tfFile <- input$tf_input
    if(is.null(tfFile)) return(NULL)
    else{
      TFpath <- paste('../data/', tfFile$name, sep='')
      assign('TFpath', TFpath, env)
    }
  })
  output$TFloaded <- reactive({
    return(!is.null(TFloading()))
  })
  outputOptions(output, 'TFloaded', suspendWhenHidden=FALSE)
  
  output$TF <- renderPrint({
    if (!is.null(TFloading()))
      cat('List of TF succesfully loaded.\n\nEnter the set of parameters and run the inference!')
  })
  
  ParamLoading <- reactive({
    RunButton <- input$run
    if(RunButton==0) return(NULL)
    else{
      return(1)
    }  })
  output$Param <- reactive({
    return(!is.null(ParamLoading()))
  })
  outputOptions(output, 'Param', suspendWhenHidden=FALSE)
  
  
  Inference <- eventReactive(input$run, {
    validate(
      need(input$Ltf >= input$Lntf, paste("Error: Please provide a Ltf value higher than ", input$Lntf, sep='')), 
      errorClass="myError"
    )
    
    if (input$run ==1) {
      old_gamma <- input$gamma
      assign("old_gamma", old_gamma, env)
      system(paste('../distrib/Alpha_Parameter.exe results/AdjMat_CLR_weights.mat', TFpath, input$gamma, sep=' '))
    }
    if (old_gamma != input$gamma)
      system(paste('../distrib/Alpha_Parameter.exe results/AdjMat_CLR_weights.mat', TFpath, input$gamma, sep=' '))
    
    ThirdTerm <- readMat('results/List_third_term.mat')$third
    output$TextParam <- renderPrint({
      cat(paste("Lntf:", input$Lntf, "-- Ltf:", input$Ltf, "-- Gamma:", input$gamma, "-- Mu:", input$mu, '\n',sep=' '))
      if (length(ThirdTerm)==0 || input$mu==0) {cat('No influence of the variational term ! \n')}
    })    
    system(paste('../distrib/Brane_Cut.exe results/EdgeList.mat', TFpath, 'results/Alpha_parameter.mat results/List_third_term.mat', input$Ltf, input$Lntf, input$mu, nb_nodes, sep=' '))
  })
  
    output$GRN <- renderPlot({
      Inference()
      Names <- readMat('results/Gene_Name.mat')
      Gene_Names <- unlist(Names$Gene.Name)
      GRNPath <- 'results/Inferred_GRN.mat'
      if (file.exists("../data/clustering.txt") == TRUE){
      clustering <- as.vector(unlist(read.table('../data/Clustering.txt', header=T)[2]))
      Test_Inference <- Visualize_Clust(Gene_Names, GRNPath, TFpath,  input$interact, clustering)
      validate(
        need(!is.null(Test_Inference), 'No inferred edges, please provide lower Ltf and/or Lntf parameter(s).'),
        errorClass="myError"
      )
      } else {
        Test_Inference <- Visualize(Gene_Names, GRNPath, TFpath,  input$interact)
        validate(
          need(!is.null(Test_Inference), 'No inferred edges, please provide lower Ltf and/or Lntf parameter(s).'),
          errorClass="myError"
        )
      }
    })
})